defmodule MessengerBot.Config do
  @moduledoc """
  App configurations
  """

  @app :messenger_bot

  @doc """
  Messenger Setup Token
  """
  def setup_token do
    @app
    |> Application.get_env(:setup_token)
    |> get_env_var()
  end

  @doc """
  Messenger App Token
  """
  def app_token do
    @app
    |> Application.get_env(:app_token)
    |> get_env_var()
  end

  def bot_name(page_id) do
    @app
    |> Application.get_env(:page_tokens)
    |> Enum.find(fn item -> Map.get(item, :id, "") == page_id end)
    |> Map.get(:name, "")
  end

  @doc """
  Messenger Page Token
  """
  def page_token(page_id) do
    @app
    |> Application.get_env(:page_tokens)
    |> Enum.find(fn item -> Map.get(item, :id, "") == page_id end)
    |> Map.get(:token, "")
    |> get_env_var()
  end

  @doc """
  Messenger Page Token
  """
  def page_token do
    @app
    |> Application.get_env(:page_tokens)
    |> List.first()
    |> Map.get(:token, "")
    |> get_env_var()
  end

  defp get_env_var({:system, name, default_val}),
    do: System.get_env(name) || default_val
  defp get_env_var(val),
    do: val
end
