defmodule MessengerBot.Service.Message do
  @moduledoc """
  Facebook Messenger 'Message' API
  """

  alias MessengerBot.Client
  alias MessengerBot.Config
  alias MessengerBot.Payload.MessageRequest

  @path "/messages"

  @doc """
  Deliver request to facebook api
  """
  @spec deliver(String.t, MessageRequest.t) :: Map.t | {:error, Map.t}
  def deliver(page_id, %MessageRequest{} = request) do
    page_token = Config.page_token(page_id)
    case Client.post(Client.url(@path, page_token), request) do
      %HTTPoison.Response{} = res ->
        Poison.decode!(res)
      {:error, err} ->
        {:error, %{details: err}}
    end
  end

  @doc """
  Deliver async request to facebook api
  """
  @spec deliver_async(String.t, MessageRequest.t) :: :ok
  def deliver_async(page_id, %MessageRequest{} = request) do
    page_token = Config.page_token(page_id)
    Client.post_async(Client.url(@path, page_token), request)
  end
end
