defmodule MessengerBot.Webhook.Supervisor do
  @moduledoc """
  A pooled supervisor for Message worker
  """

  defmacro __using__(opts) do
    quote do
      use Supervisor

      @pool_name unquote(opts)[:pool_name]
      @pool_worker unquote(opts)[:pool_worker]
      @pool_size Application.get_env(:messenger_bot, :"#{@pool_name}_pool_size", 1)
      @pool_max_overflow Application.get_env(:messenger_bot,
        :"#{@pool_name}_pool_max_overflow", 1)

      @doc false
      def start_link,
        do: Supervisor.start_link(__MODULE__, [], name: __MODULE__)

      @doc false
      @spec init([]) :: no_return()
      def init([]) do
        worker_pool_options = [
          name: {:local, @pool_name},
          worker_module: @pool_worker,
          size: @pool_size,
          max_overflow: @pool_max_overflow
        ]

        children = [
          :poolboy.child_spec(@pool_name, worker_pool_options, [])
        ]

        opts = [strategy: :one_for_one, name: __MODULE__]

        supervise(children, opts)
      end
    end
  end
end
