defmodule MessengerBot.Webhook.Worker do
  @moduledoc """
  Worker
  """

  defmacro __using__(opts) do
    quote do
      use GenServer
      alias MessengerBot.Webhook.Processor.Log
      alias MessengerBot.Payload.Process, as: ProcessPayload

      @name unquote(opts)[:pool_name]
      @model unquote(opts)[:model]
      @processor Application.get_env(:messenger_bot, :"#{@name}_processor", Log)

      ## Public api

      @doc """
      Async perform data in workers.
      """
      @spec perform(ProcessPayload.t) :: :ok
      def perform(item) do
        :poolboy.transaction(@name, fn(worker) ->
          GenServer.cast(worker, {:run, item})
        end)
        :ok
      end

      ## Callbacks

      @doc false
      def start_link(_opts),
        do: GenServer.start_link(__MODULE__, [])

      @doc false
      def init(_opts),
        do: {:ok, []}

      @doc false
      def handle_cast({:run, item}, state) do
        @processor.run(item)
        {:noreply, state}
      end
    end
  end
end
